/***********************************************************************/
/* TODO: update the document header for SDL !!!  -- with a little help from the CQ3A source code */
/* FreeVR OpenGL tutorial example 0: ex0_sdl.c                         */
/* Last tested with: N/A (since this is not a VR program)              */
/* Last Modified: March 23, 2004                                       */
/* URL: http://freevr.org                                              */
/***********************************************************************/
/*                                                                     */
/* This example code demonstrates an OpenGL application program that   */
/*   displays a blue cube and a red pyramid in a simple X window.      */
/*                                                                     */
/*                                                                     */
/* The features of this example include:                               */
/*   - Opens an X window                                               */
/*   - Creates an OpenGL viewport                                      */
/*   - Renders a simple static world                                   */
/*                                                                     */
/*                                                                     */
/* Considerations:                                                     */
/*   - A lot of code is necessary to open the GLX window (X w/ OpenGL) */
/*   - The main loop would typically watch for many more X Events      */
/*   - This is not a VR program                                        */
/*                                                                     */
/* This application uses the files:                                    */
/*   - shapes.c  (Bill Sherman's simple OpenGL shapes)                 */
/*                                                                     */
/* Copyright 2010, William R. Sherman, All Rights Reserved.            */
/*   In particular, the right to republish this file in whole or in    */
/*   part, in any form (including electronic forms) is prohibited      */
/*   without written consent of the copyright holder.                  */
/***********************************************************************/
#include <stdio.h>	/* needed for printf(), etc. */
#include <stdlib.h>	/* needed for exit() */

#include <GL/gl.h>	/* needed for all OpenGL functions & values */
#include <SDL/SDL.h>	/* needed for the SDL library functions */

void	exit(int status);
void	draw_world(void);			/* graphically render the world    */

#define WINDOW_WIDTH	300
#define WINDOW_HEIGHT	300


/***********************************************************************/
main(int argc, char* argv[])
{
	int		done = 0;
	int		bits_per_pixel = 32;
	Uint32		videoFlags;
	SDL_Event	event;
	int		key;

	/***********************************/
	/*** initialize GL-SDL routines  ***/
	/***********************************/

	/* Initialize SDL for video output */
	if (SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
		exit(1);
	}

	/* Create an OpenGL screen */
	videoFlags = SDL_OPENGL;
	if (SDL_SetVideoMode(WINDOW_WIDTH, WINDOW_HEIGHT, bits_per_pixel, videoFlags) == NULL ) {
		fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
		SDL_Quit();
		exit(2);
	}

	SDL_ShowCursor(0);

	/*** configure the OpenGL context for rendering ***/
	glEnable(GL_DEPTH_TEST);/* enable depth buffering */
	glDepthFunc(GL_LESS);	/* pedantic, GL_LESS is the default */
	glClearDepth(1.0);	/* pedantic, 1.0 is the default */
	/* frame buffer clears should be to black */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	/* set up projection transform */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-1.0, 1.0, -1.0, 1.0, 1.0, 10.0);
	/* establish initial viewport */
	glViewport(0, 0, 300, 300);	/* pedantic, full window size is default viewport */

	/***************************/
	/*** do world simulation ***/
	/***************************/
	while (!done) {
		/*** Handle projection ***/
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glTranslatef(0.0, -3.0, 0.0);

		/*** Render the world ***/
		draw_world();

		/*** Swap buffers ***/
		SDL_GL_SwapBuffers();

		/*** Get and handle inputs ***/
		while (SDL_PollEvent(&event)) {
			/* do stuff here to handle the events */
			switch (event.type) {
			case SDL_QUIT:
				done = 1;
				break;
#if 1 /* typically I wouldn't include this "fancy" feature in the ex0 example, but SDL masks the interrupt signal */
			case SDL_KEYDOWN:
				key = event.key.keysym.sym;
				if (key == SDLK_ESCAPE || key == SDLK_q)
					done = 1;
#endif
			}
		}
	}

	SDL_Quit();

	/*********************/
	/*** close up shop ***/
	/*********************/
	exit(0);
}



/* ----------------8<-----------------8<-----------------8<----------------*/
/* In a non-example applicatation, the following would be a separate file. */

		/**************************/
		/**************************/
		/** The Graphics section **/
		/**************************/
		/**************************/

/* functions from shapes.c */
void	draw_cube();
void	draw_pyramid();


/*************************************************************/
/* draw_world(): Render the two objects in the world -- the  */
/*   blue cube, and red pyramid -- such that they are on the */
/*   XZ plane (typically the floor), and 6 feet forward      */
/*   (typically on the other side of a CAVE's front screen). */
/*************************************************************/
void draw_world(void)
{
	/**************************************/
	/* set the polygon shading parameters */
	glShadeModel(GL_SMOOTH);

	/**************************************/
	/* clear the screen -- very important */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClearDepth(1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/************************/
	/* draw all the objects */

	/* a blue cube */
	glColor3ub(100, 100, 255);
	glTranslatef(-2.0, 1.0, -6.0);
	draw_cube();
	glColor3ub(0, 0, 0);
	draw_cube_outline();

	/* a red pyramid */
	glColor3ub(255, 100, 100);
	glTranslatef(4.0, 0.0, 0.0);
	draw_pyramid();
	glColor3ub(0, 0, 0);
	draw_pyramid_outline();
}

