/***********************************************************************/
/* FreeVR OpenGL tutorial example 1: ex1_bare.c                        */
/* Last tested with: FreeVR version 0.6a                               */
/* Last Modified: March 23, 2004                                       */
/* URL: http://freevr.org                                              */
/***********************************************************************/
/* This example code demonstrates a VR application program that        */
/*   initializes the VR library routines, and renders a blue cube and  */
/*   a red pyramid on the floor near the front wall.                   */
/*                                                                     */
/*                                                                     */
/* The features of this example include:                               */
/*   - FreeVR Library functions, variables, and macros:                */
/*      * vrStart()                                                    */
/*      * vrFunctionSetCallback(VRFUNC_DISPLAY, ...)                   */
/*      * vrExit()                                                     */
/*   - A procedure that renders a simple static world                  */
/*                                                                     */
/*                                                                     */
/* Considerations:                                                     */
/*   - No interaction at all                                           */
/*   - Program self-terminates after 10 seconds                        */
/*   - Does not handle everything in a civilized manner                */
/*                                                                     */
/* This application uses the files:                                    */
/*   - shapes.c  (Bill Sherman's simple OpenGL shapes)                 */
/*                                                                     */
/* Copyright 2010, William R. Sherman, All Rights Reserved.            */
/*   In particular, the right to republish this file in whole or in    */
/*   part, in any form (including electronic forms) is prohibited      */
/*   without written consent of the copyright holder.                  */
/***********************************************************************/
#include <stdio.h>	/* needed for printf(), etc. */
#include <stdlib.h>	/* needed for exit() */

#include <GL/gl.h>	/* needed for all OpenGL functions & values */

#include "freevr.h"	/* needed for FreeVR functions & values */


/*************************************************************************************/
/* These function declarations are very important -- they will be used as arguments. */
void	draw_world(void);			/* graphically render the world    */


/***********************************************************************/
main()
{

	/**********************************/
	/*** initialize FreeVR routines ***/
	/**********************************/
	vrStart();

	/***************************************/
	/* set up the FreeVR library functions */
	vrFunctionSetCallback(VRFUNC_DISPLAY, vrCallbackCreate(draw_world, 0));

	/***************************/
	/*** do world simulation ***/
	/***************************/
	/* do stuff here -- or not */
	vrSleep(10000000);	/* 10 seconds */

	/*********************/
	/*** close up shop ***/
	/*********************/
	vrExit();

	exit(0);
}



/* ----------------8<-----------------8<-----------------8<----------------*/
/* In a non-example applicatation, the following would be a separate file. */

		/**************************/
		/**************************/
		/** The Graphics section **/
		/**************************/
		/**************************/

/* functions from shapes.c */
void	draw_cube();
void	draw_pyramid();


/*************************************************************/
/* draw_world(): Render the two objects in the world -- the  */
/*   blue cube, and red pyramid -- such that they are on the */
/*   XZ plane (typically the floor), and 6 feet forward      */
/*   (typically on the other side of a CAVE's front screen). */
/*************************************************************/
void draw_world(void)
{
	/**************************************/
	/* set the polygon shading parameters */
	glShadeModel(GL_SMOOTH);

	/**************************************/
	/* clear the screen -- very important */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClearDepth(1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/************************/
	/* draw all the objects */

	/* a blue cube */
	glColor3ub(100, 100, 255);
	glTranslatef(-2.0, 1.0, -6.0);
	draw_cube();

	/* a red pyramid */
	glColor3ub(255, 100, 100);
	glTranslatef(4.0, 0.0, 0.0);
	draw_pyramid();
}

